@@ -6,7 +6,7 @@ Python. It also provides an index for other astronomy packages and tools for
 managing them.
 """
 
-#this indicates whether or not we are in astropy's setup.py
+# this indicates whether or not we are in astropy's setup.py
 try:
     _ASTROPY_SETUP_
 except NameError:
@@ -29,11 +29,24 @@ except ImportError:
     # TODO: Issue a warning using the logging framework
     __githash__ = ''
 
+
+import logging
+
+
+# The location of the online documentation for astropy
+# This location will normally point to the current released version of astropy
+if 'dev' in __version__:
+    online_docs_root = 'http://docs.astropy.org/en/latest/'
+else:
+    online_docs_root = 'http://docs.astropy.org/en/{0}/'.format(__version__)
+
+
 # set up the test command
 def _get_test_runner():
     from .tests.helper import TestRunner
     return TestRunner(__path__[0])
 
+
 def test(package=None, test_path=None, args=None, plugins=None,
          verbose=False, pastebin=None, remote_data=False, pep8=False,
          pdb=False, coverage=False, open_files=False):
@@ -103,27 +116,47 @@ def test(package=None, test_path=None, args=None, plugins=None,
         remote_data=remote_data, pep8=pep8, pdb=pdb,
         coverage=coverage, open_files=open_files)
 
+
+# Use the root logger as a dummy log before initilizing Astropy's logger
+log = logging.getLogger()
+
 # if we are *not* in setup mode, import the logger and possibly populate the
 # configuration file with the defaults
 if not _ASTROPY_SETUP_:
-    from .logger import log
+    from .logger import _init_log
     from . import config
 
     import os
+    import sys
     from warnings import warn
 
+    try:
+        from .utils import _compiler
+    except ImportError:
+        if os.path.exists('setup.py'):
+            log.error('You appear to be trying to import astropy from within '
+                      'a source checkout; please run `./setup.py develop` or '
+                      '`./setup.py build_ext --inplace` first so that '
+                      'extension modules can be compiled and made importable.')
+            sys.exit(1)
+        else:
+            # Outright broken installation; don't be nice.
+            raise
+
+
+
     # add these here so we only need to cleanup the namespace at the end
     config_dir = None
 
-    if not os.environ.get('ASTROPY_SKIP_CONFIG_UPDATE', False):
-        config_dir = os.path.dirname(__file__)
-        try:
-            config.configuration.update_default_config(__package__, config_dir)
-        except config.configuration.ConfigurationDefaultMissingError as e:
-            wmsg = (e.args[0] + " Cannot install default profile. If you are "
-                    "importing from source, this is expected.")
-            warn(config.configuration.ConfigurationDefaultMissingWarning(wmsg))
-            del e
+    log = _init_log()
 
-    del os, warn, config_dir  # clean up namespace
+    config_dir = os.path.dirname(__file__)
+    try:
+        config.configuration.update_default_config(__package__, config_dir)
+    except config.configuration.ConfigurationDefaultMissingError as e:
+        wmsg = (e.args[0] + " Cannot install default profile. If you are "
+                "importing from source, this is expected.")
+        warn(config.configuration.ConfigurationDefaultMissingWarning(wmsg))
+        del e
 
+    del _init_log, os, warn, config_dir  # clean up namespace
