@@ -64,7 +64,7 @@ def test_create_angles():
     a3 = Angle("54:07:26.832", unit=u.degree)
     a4 = Angle("54.12412 deg")
     a5 = Angle("54.12412 degrees")
-    a6 = Angle("54.12412°") # because we like Unicode
+    a6 = Angle(u"54.12412°") # because we like Unicode
     a7 = Angle((54, 7, 26.832), unit=u.degree)
     # (deg,min,sec) *tuples* are acceptable, but lists/arrays are *not*
     # because of the need to eventually support arrays of coordinates
@@ -82,7 +82,7 @@ def test_create_angles():
         #raises an exception because this is ambiguous
 
     with raises(ValueError):
-    	a13 = Angle(12.34, unit="not a unit")
+        a13 = Angle(12.34, unit="not a unit")
 
     a14 = Angle("12h43m32") # no trailing 's', but unambiguous
 
@@ -236,7 +236,7 @@ def test_angle_bounds():
     a10.degrees == 0
 
     with raises(AttributeError):
-	    a10.bounds = (0,34)
+        a10.bounds = (0,34)
 
 def test_angle_convert():
     """
@@ -516,7 +516,7 @@ def test_create_coordinate():
         c = ICRSCoordinates("12 34 56  -56 23 21") # ambiguous
 
     with raises(TypeError):
-	    c = ICRSCoordinates() # not allowed
+        c = ICRSCoordinates() # not allowed
 
     c = ICRSCoordinates(ra="12 43 12", dec=dec, unit=(u.hour, u.hour))
 
@@ -700,7 +700,7 @@ def test_distances():
         Distance(12)
 
     # standard units are pre-defined
-    npt.assert_almost_equal(distance.lightyear, 39.13876728075561)
+    npt.assert_almost_equal(distance.lightyear, 39.138765325702551)
     npt.assert_almost_equal(distance.km, 370281309776063.0)
 
     # Coordinate objects can be assigned a distance object, giving them a full
@@ -786,3 +786,14 @@ def test_distances_scipy():
 
     assert abs(d4.z - 0.23) < 1e-8  # redshift, assuming "current" cosmology
     assert abs(d5.compute_z(WMAP5) - 0.23) < 1e-8 # specifying a cosmology possible
+
+
+def test_unicode():
+    """
+    This test could only possibly fail when `sys.getdefaultencoding()`
+    is not `utf-8` -- but given a recent fix, is expected to pass on
+    all platforms.
+    """
+    from .. import ICRSCoordinates
+    u = unicode("12h46m11.086s -00d30m11.99s")
+    c = ICRSCoordinates(u)
