@@ -4,10 +4,10 @@ import operator
 import sys
 import warnings
 
-from functools import reduce
-
 import numpy as np
 
+from functools import reduce
+
 from .base import DELAYED, ExtensionHDU
 from .image import _ImageBaseHDU, ImageHDU
 from .table import BinTableHDU
@@ -34,9 +34,15 @@ DEFAULT_HCOMP_SMOOTH = 0
 DEFAULT_BLOCK_SIZE = 32
 DEFAULT_BYTE_PIX = 4
 
+
 # CFITSIO version-specific features
 if COMPRESSION_SUPPORTED:
-    CFITSIO_SUPPORTS_GZIPDATA = compression.CFITSIO_VERSION >= 3.28
+    try:
+        CFITSIO_SUPPORTS_GZIPDATA = compression.CFITSIO_VERSION >= 3.28
+    except AttributeError:
+        # This generally shouldn't happen unless running setup.py in an
+        # environment where an old build of pyfits exists
+        CFITSIO_SUPPORTS_GZIPDATA = True
 
 
 class CompImageHeader(Header):
@@ -455,9 +461,10 @@ class CompImageHDU(BinTableHDU):
         # Create the additional columns required for floating point
         # data and calculate the width of the output table.
 
-        if self._image_header['BITPIX'] < 0:
+        if self._image_header['BITPIX'] < 0 and quantizeLevel != 0.0:
             # floating point image has 'COMPRESSED_DATA',
-            # 'UNCOMPRESSED_DATA', 'ZSCALE', and 'ZZERO' columns.
+            # 'UNCOMPRESSED_DATA', 'ZSCALE', and 'ZZERO' columns (unless using
+            # lossless compression, per CFITSIO)
             ncols = 4
 
             # CFITSIO 3.28 and up automatically use the GZIP_COMPRESSED_DATA
@@ -1199,8 +1206,6 @@ class CompImageHDU(BinTableHDU):
         Compress the image data so that it may be written to a file.
         """
 
-        tilesizes = []
-
         # Check to see that the image_header matches the image data
         image_bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
 
@@ -1211,12 +1216,6 @@ class CompImageHDU(BinTableHDU):
             self.shape != self.data.shape):
             self.updateHeaderData(self.header)
 
-        # Create lists to hold the number of pixels along each axis of
-        # the image data and the number of pixels in each tile of the
-        # compressed image.
-        for idx in range(self._header['ZNAXIS']):
-            tilesizes.append(self._header['ZTILE' + str(idx + 1)])
-
         # put data in machine native byteorder on little endian machines
         # for handing off to the compression code
         if sys.byteorder == 'little':
@@ -1242,32 +1241,8 @@ class CompImageHDU(BinTableHDU):
         if should_swap:
             self.data.byteswap(True)
 
-        # Estimate memory needed for the compressed data and allocate it;
-        # CFITSIO will handle growing allocated memory if necessary, after
-        # which the pyfits.compression module will updating self.compData to
-        # use the newly reallocated buffer
-        if self._header['ZCMPTYPE'] == 'RICE_1':
-            rice_blocksize = self._header['ZVAL1']
-        else:
-            rice_blocksize = 0
-        maxtilelen = reduce(operator.mul, tilesizes, 1)
         nrows = self._header['NAXIS2']
         tbsize = self._header['NAXIS1'] * nrows
-        max_elem = compression.calc_max_elem(self._header['ZCMPTYPE'],
-                                             maxtilelen,
-                                             self._header['ZBITPIX'],
-                                             rice_blocksize)
-        dataspan = tbsize + (nrows * max_elem)
-        if dataspan < BLOCK_SIZE:
-            # We must a full FITS block at a minimum
-            dataspan = BLOCK_SIZE
-        else:
-            # Still make sure to pad out to a multiple of 2880 byte blocks
-            # otherwise CFITSIO can get read errors when it tries to read
-            # a partial block that goes past the end of the file
-            dataspan += _pad_length(dataspan)
-        self.compData = np.empty((dataspan,), dtype=np.byte)
-        self.compData[:tbsize] = 0
 
         self._header['PCOUNT'] = 0
         if 'THEAP' in self._header:
@@ -1281,7 +1256,7 @@ class CompImageHDU(BinTableHDU):
             # self.compData, and writes directly to it
             # compress_hdu returns the size of the heap for the written
             # compressed image table
-            heapsize = compression.compress_hdu(self)
+            heapsize, self.compData = compression.compress_hdu(self)
         finally:
             # if data was byteswapped return it to its original order
             if should_swap:
@@ -1339,10 +1314,10 @@ class CompImageHDU(BinTableHDU):
 
         # Update TFORM for variable length columns.
         for idx in range(self.compData._nfields):
-            format = self.compData._coldefs.formats[idx]
+            format = self.compData._coldefs._recformats[idx]
             if isinstance(format, _FormatP):
-                max = self.compData.field(idx).max
-                format = _FormatP(format, repeat=format.repeat, max=max)
+                _max = self.compData.field(idx).max
+                format = _FormatP(format.dtype, repeat=format.repeat, max=_max)
                 self._header['TFORM' + str(idx + 1)] = format.tform
         # Insure that for RICE_1 that the BLOCKSIZE and BYTEPIX cards
         # are present and set to the hard coded values used by the
@@ -1418,18 +1393,18 @@ class CompImageHDU(BinTableHDU):
                     _scale = 1
                     _zero = 0
                 else:
-                    min = np.minimum.reduce(self.data.flat)
-                    max = np.maximum.reduce(self.data.flat)
+                    _min = np.minimum.reduce(self.data.flat)
+                    _max = np.maximum.reduce(self.data.flat)
                     self.data.shape = dims
 
                     if _type == np.uint8:  # uint8 case
-                        _zero = min
-                        _scale = (max - min) / (2. ** 8 - 1)
+                        _zero = _min
+                        _scale = (_max - _min) / (2. ** 8 - 1)
                     else:
-                        _zero = (max + min) / 2.
+                        _zero = (_max + _min) / 2.
 
                         # throw away -2^N
-                        _scale = (max - min) / (2. ** (8 * _type.bytes) - 2)
+                        _scale = (_max - _min) / (2. ** (8 * _type.bytes) - 2)
 
         # Do the scaling
         if _zero != 0:
