@@ -76,7 +76,7 @@ def handle_options(argv=None):
             try:
                 values = [v.strip() for v in open(value, 'r').readlines()]
                 setattr(parser.values, option.dest, values)
-            except IOError, e:
+            except IOError as e:
                 log.warning('reading %s for %s failed: %s; ignoring this '
                             'argument' % (value, opt, e))
         else:
