@@ -945,31 +945,45 @@ class TestImageFunctions(FitsTestCase):
                 hdul2[0].data[0] = 0
                 assert (hdul[1].data == hdul2[0].data).all()
 
-    @pytest.mark.skipif("sys.platform.startswith('win')")
-    def test_insufficient_compression_allocation(self):
-        """
-        Test the scenario where not enough space is allocated to hold the
-        compressed data, and a realloc is necessary.
+    def test_lossless_gzip_compression(self):
+        """Regression test for #198."""
 
-        For some reason this test *sometimes* breaks Jenkins when run on
-        Windows. It passes when the test suite is run outside Jenkins. I intend
-        to track down the cause of this but in the meantime it's disabled on
-        Windows.  See Astropy #507.
-        """
+        noise = np.random.normal(size=(1000, 1000))
 
-        data = np.arange(10000, dtype='int32').reshape(100, 100)
-        hdu = fits.CompImageHDU(data=data)
-        old_compress_hdu = fits.compression.compress_hdu
+        chdu1 = fits.CompImageHDU(data=noise, compressionType='GZIP_1')
+        # First make a test image with lossy compression and make sure it
+        # wasn't compressed perfectly.  This shouldn't happen ever, but just to
+        # make sure the test non-trivial.
+        chdu1.writeto(self.temp('test.fits'))
 
-        def hacked_compress_hdu(hdu):
-            # Long enough to hold the table, but not enough for the full heap
-            hdu.compData = np.zeros((2880,), dtype=np.uint8)
-            return old_compress_hdu(hdu)
+        with fits.open(self.temp('test.fits')) as h:
+            assert np.abs(noise - h[1].data).max() > 0.0
 
-        fits.compression.compress_hdu = hacked_compress_hdu
+        del h
 
-        try:
-            hdu.updateCompressedData()
-            assert (data == fits.compression.decompress_hdu(hdu)).all()
-        finally:
-            fits.compression.compress_hdu = old_compress_hdu
+        chdu2 = fits.CompImageHDU(data=noise, compressionType='GZIP_1',
+                                    quantizeLevel=0.0)  # No quantization
+        with ignore_warnings():
+            chdu2.writeto(self.temp('test.fits'), clobber=True)
+
+        with fits.open(self.temp('test.fits')) as h:
+            assert (noise == h[1].data).all()
+
+    def test_compression_column_tforms(self):
+        """Regression test for #199."""
+
+        # Some interestingly tiled data so that some of it is quantized and
+        # some of it ends up just getting gzip-compressed
+        data2 = ((np.arange(1, 8, dtype=np.float32) * 10)[:, np.newaxis] +
+                np.arange(1, 7))
+        np.random.seed(1337)
+        data1 = np.random.uniform(size=(6 * 4, 7 * 4))
+        data1[:data2.shape[0], :data2.shape[1]] = data2
+        chdu = fits.CompImageHDU(data1, compressionType='RICE_1',
+                                   tileSize=(6, 7))
+        chdu.writeto(self.temp('test.fits'))
+
+        with fits.open(self.temp('test.fits'),
+                         disable_image_compression=True) as h:
+            assert h[1].header['TFORM1'] == '1PB(30)'
+            assert h[1].header['TFORM2'] == '1PB(359)'
