@@ -138,7 +138,8 @@ def parse_vowarning(line):
             result['is_warning'] = (warning[0].upper() == 'W')
             result['is_exception'] = not result['is_warning']
             result['number'] = int(match.group('warning')[1:])
-            result['doc_url'] = "vo/api_exceptions.html#%s" % warning.lower()
+            result['doc_url'] = "io/votable/api_exceptions.html#{0}".format(
+                warning.lower())
         else:
             result['is_warning'] = False
             result['is_exception'] = False
@@ -999,6 +1000,15 @@ class W50(VOTableSpecWarning):
     default_args = ('x',)
 
 
+class W51(VOTableSpecWarning):
+    """
+    The integer value is out of range for the size of the field.
+    """
+
+    message = "Value '%s' is out of range for a %s integer field"
+    default_args = ('x', 'n-bit')
+
+
 class E01(VOWarning, ValueError):
     """
     The size specifier for a ``char`` or ``unicode`` field must be
