@@ -6,6 +6,7 @@ setup/build/packaging that are useful to astropy as a whole.
 
 from __future__ import absolute_import, print_function
 
+import collections
 import errno
 import imp
 import os
@@ -29,6 +30,8 @@ from setuptools.command.register import register as SetuptoolsRegister
 
 from .tests.helper import astropy_test
 from .utils import silence
+from .utils.compat.misc import invalidate_caches
+from .utils.misc import walk_skip_hidden
 
 
 try:
@@ -46,6 +49,9 @@ except ImportError:
     HAVE_SPHINX = False
 
 
+PY3 = sys.version_info[0] >= 3
+
+
 _adjusted_compiler = False
 def adjust_compiler(package):
     """
@@ -478,6 +484,11 @@ def generate_build_ext_command(release):
                                                       extension.name))
                             raise IOError(errno.ENOENT, msg, cfn)
 
+        if orig_run is not None:
+            # This should always be the case for a correctly implemented
+            # distutils command.
+            orig_run(self)
+
         # Update cython_version.py if building with Cython
         try:
             from .version import cython_version
@@ -490,21 +501,23 @@ def generate_build_ext_command(release):
                 f.write('# Generated file; do not modify\n')
                 f.write('cython_version = {0!r}\n'.format(self.uses_cython))
 
-            self.copy_file(cython_py, os.path.join(self.build_lib, cython_py),
-                           preserve_mode=False)
+            if os.path.isdir(self.build_lib):
+                # The build/lib directory may not exist if the build_py command
+                # was not previously run, which may sometimes be the case
+                self.copy_file(cython_py,
+                               os.path.join(self.build_lib, cython_py),
+                               preserve_mode=False)
 
-        if orig_run is not None:
-            # This should always be the case for a correctly implemented
-            # distutils command.
-            orig_run(self)
+            invalidate_caches()
 
-        if not self.distribution.is_pure():
+        if not self.distribution.is_pure() and os.path.isdir(self.build_lib):
             # Finally, generate the default astropy.cfg; this can only be done
-            # after extension modules are built as some extension modules include
-            # config items.  We only do this if it's not pure python, though,
-            # because if it is, we already did it in build_py
-            default_cfg = generate_default_config(os.path.abspath(self.build_lib),
-                                                  self.distribution.packages[0])
+            # after extension modules are built as some extension modules
+            # include config items.  We only do this if it's not pure python,
+            # though, because if it is, we already did it in build_py
+            default_cfg = generate_default_config(
+                    os.path.abspath(self.build_lib),
+                    self.distribution.packages[0])
             if default_cfg:
                 default_cfg = os.path.relpath(default_cfg)
                 self.copy_file(default_cfg,
@@ -528,8 +541,9 @@ class AstropyBuildPy(SetuptoolsBuildPy):
         if self.distribution.is_pure():
             # Generate the default astropy.cfg - we only do this here if it's
             # pure python.  Otherwise, it'll happen at the end of build_exp
-            default_cfg = generate_default_config(os.path.abspath(self.build_lib),
-                                                  self.distribution.packages[0])
+            default_cfg = generate_default_config(
+                    os.path.abspath(self.build_lib),
+                    self.distribution.packages[0])
             if default_cfg:
                 default_cfg = os.path.relpath(default_cfg)
                 self.copy_file(default_cfg,
@@ -546,20 +560,24 @@ def generate_default_config(build_lib, package):
     else:
         log.info('generating default {0}.cfg file'.format(package))
 
-    env = os.environ.copy()
-    env['ASTROPY_SKIP_CONFIG_UPDATE'] = 'True'
+    if PY3:
+        builtins = 'builtins'
+    else:
+        builtins = '__builtin__'
 
     subproccode = (
+        'import {builtins};{builtins}._ASTROPY_SETUP_ = True;'
         'from astropy.config.configuration import generate_all_config_items;'
         'generate_all_config_items({pkgnm!r}, True, filename={filenm!r})')
-    subproccode = subproccode.format(pkgnm=package,
+    subproccode = subproccode.format(builtins=builtins,
+                                     pkgnm=package,
                                      filenm=os.path.abspath(filename))
 
     # Note that cwd=build_lib--we're importing astropy from the build/ dir
     # but using the astropy/ source dir as the config directory
     proc = subprocess.Popen([sys.executable, '-c', subproccode],
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                             cwd=build_lib, env=env)
+                             cwd=build_lib)
     stdout, stderr = proc.communicate()
 
     if proc.returncode == 0 and os.path.exists(filename):
@@ -567,7 +585,7 @@ def generate_default_config(build_lib, package):
     else:
         msg = ('Generation of default configuration item failed! Stdout '
                'and stderr are shown below.\n'
-               'Stdout:\n{stdout}\nStderr:\n{stderr}')
+               'Stdout:\n{stdout}\nStderr:\n{stderr}').decode('UTF-8')
         log.error(msg.format(stdout=stdout.decode('UTF-8'),
                              stderr=stderr.decode('UTF-8')))
 
@@ -959,8 +977,7 @@ def iter_setup_packages(srcdir):
         `modname` is the module name for the ``setup_package.py`` modules.
 
     """
-
-    for root, dirs, files in os.walk(srcdir):
+    for root, dirs, files in walk_skip_hidden(srcdir):
         if 'setup_package.py' in files:
             filename = os.path.join(root, 'setup_package.py')
             module = import_file(filename)
@@ -980,7 +997,7 @@ def iter_pyx_files(srcdir):
         the .pyx file.
 
     """
-    for dirpath, dirnames, filenames in os.walk(srcdir):
+    for dirpath, dirnames, filenames in walk_skip_hidden(srcdir):
         modbase = dirpath.replace(os.sep, '.')
         for fn in filenames:
             if fn.endswith('.pyx'):
@@ -1271,8 +1288,27 @@ def add_legacy_alias(old_package, new_package, equiv_version, extras={}):
     return (old_package, shim_dir)
 
 
-def pkg_config(packages, default_libraries, include_dirs, library_dirs,
-               libraries):
+class DistutilsExtensionArgs(collections.defaultdict):
+    """
+    A special dictionary whose default values are the empty list.
+
+    This is useful for building up a set of arguments for
+    `distutils.Extension` without worrying whether the entry is
+    already present.
+    """
+    def __init__(self, *args, **kwargs):
+        def default_factory():
+            return []
+
+        super(DistutilsExtensionArgs, self).__init__(
+            default_factory, *args, **kwargs)
+
+    def update(self, other):
+        for key, val in other.items():
+            self[key].extend(val)
+
+
+def pkg_config(packages, default_libraries):
     """
     Uses pkg-config to update a set of distutils Extension arguments
     to include the flags necessary to link against the given packages.
@@ -1288,23 +1324,27 @@ def pkg_config(packages, default_libraries, include_dirs, library_dirs,
     default_libraries : list of str
         A list of library names to use if the pkg-config lookup fails.
 
-    include_dirs : list of str
-        A list of include directories that will be updated to include
-        the results from pkg-config.
+    Returns
+    -------
+    config : dict
+        A dictionary containing keyword arguments to
+        `distutils.Extension`.  These entries include:
 
-    library_dirs : list of str
-        A list of library directories that will be updated to include
-        the results from pkg-config.
-
-    libraries : list of str
-        A list of library names that will be updated to include the
-        results from pkg-config, or if pkg-config fails, updated from
-        *default_libraries*.
+        - ``include_dirs``: A list of include directories
+        - ``library_dirs``: A list of library directories
+        - ``libraries``: A list of libraries
+        - ``define_macros``: A list of macro defines
+        - ``undef_macros``: A list of macros to undefine
+        - ``extra_compile_args``: A list of extra arguments to pass to
+          the compiler
     """
 
-    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries'}
+    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries',
+                '-D': 'define_macros', '-U': 'undef_macros'}
     command = "pkg-config --libs --cflags {0}".format(' '.join(packages)),
 
+    result = DistutilsExtensionArgs()
+
     try:
         pipe = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
         output = pipe.communicate()[0].strip()
@@ -1316,7 +1356,7 @@ def pkg_config(packages, default_libraries, include_dirs, library_dirs,
             "  output: {0}".format(e.output)
             ]
         log.warn('\n'.join(lines))
-        libraries.extend(default_libraries)
+        result['libraries'].extend(default_libraries)
     else:
         if pipe.returncode != 0:
             lines = [
@@ -1325,10 +1365,26 @@ def pkg_config(packages, default_libraries, include_dirs, library_dirs,
                 "This may cause the build to fail below."
                 ]
             log.warn('\n'.join(lines))
-            libraries.extend(default_libraries)
+            result['libraries'].extend(default_libraries)
         else:
             for token in output.split():
-                locals()[flag_map.get(token[:2])].append(token[2:])
+                # It's not clear what encoding the output of
+                # pkg-config will come to us in.  It will probably be
+                # some combination of pure ASCII (for the compiler
+                # flags) and the filesystem encoding (for any argument
+                # that includes directories or filenames), but this is
+                # just conjecture, as the pkg-config documentation
+                # doesn't seem to address it.
+                arg = token[:2].decode('ascii')
+                value = token[2:].decode(sys.getfilesystemencoding())
+                if arg in flag_map:
+                    if arg == '-D':
+                        value = tuple(value.split('=', 1))
+                    result[flag_map[arg]].append(value)
+                else:
+                    result['extra_compile_args'].append(value)
+
+    return result
 
 
 def add_external_library(library):
@@ -1378,7 +1434,8 @@ def filter_packages(packagenames):
     Removes some packages from the package list that shouldn't be
     installed on the current version of Python.
     """
-    if sys.version_info[0] >= 3:
+
+    if PY3:
         exclude = '_py2'
     else:
         exclude = '_py3'
