@@ -151,6 +151,14 @@ class TestReverse():
         assert np.all(t2['col0'] == np.array([1, 2, 3]))
         assert np.all(t2['col1'] == np.array(['a', 'b', 'cc']))
 
+    def test_reverse_big(self):
+        x = np.arange(10000)
+        y = x + 1
+        t = Table([x, y], names=('x', 'y'))
+        t.reverse()
+        assert np.all(t['x'] == x[::-1])
+        assert np.all(t['y'] == y[::-1])
+
 
 @pytest.mark.usefixtures('set_global_Table')
 class TestColumnAccess():
@@ -533,6 +541,8 @@ class TestRename(SetupData):
         t.rename_column('b', 'a')
         assert t.columns.keys() == ['c', 'a']
         assert t._data.dtype.names == ('c', 'a')
+        if t.masked:
+            assert t._data.mask.dtype.names == ('c', 'a')
         assert np.all(t['c'] == np.array([1, 2, 3]))
         assert np.all(t['a'] == np.array([4, 5, 6]))
 
@@ -562,6 +572,16 @@ class TestSort():
         assert np.all(t['a'] == np.array([3, 1, 2]))
         assert np.all(t['b'] == np.array([4, 5, 6]))
 
+    def test_single_big(self):
+        """Sort a big-ish table with a non-trivial sort order"""
+        x = np.arange(10000)
+        y = np.sin(x)
+        t = Table([x, y], names=('x', 'y'))
+        t.sort('y')
+        idx = np.argsort(y)
+        assert np.all(t['x'] == x[idx])
+        assert np.all(t['y'] == y[idx])
+
     def test_multiple(self):
         t = Table()
         t.add_column(Column(name='a', data=[2, 1, 3, 2, 3, 1]))
