@@ -814,7 +814,8 @@ class NamedUnit(UnitBase):
                 if st in self._namespace:
                     raise ValueError(
                         "Object with name {0!r} already exists "
-                        "in namespace".format(st))
+                        "in namespace ({1})".format(
+                            st, self._namespace[st].__doc__))
                 self._namespace[st] = self
 
             self._registry[st] = self
@@ -1217,6 +1218,13 @@ class CompositeUnit(UnitBase):
     _simplify.__doc__ = UnitBase._simplify.__doc__
 
     def decompose(self, bases=[]):
+        for base in self.bases:
+            if (not isinstance(base, IrreducibleUnit) or
+                    (len(bases) and base not in bases)):
+                break
+        else:
+            return self
+
         x = CompositeUnit(self.scale, self.bases, self.powers)
         x._expand_and_gather(True, bases=bases)
         return x
@@ -1279,16 +1287,12 @@ def _add_prefixes(u, excludes=[], register=False):
         namespace.  Default is `False`.
     """
     for short, long, factor in si_prefixes:
-        exclude = False
-        for prefix in short:
-            if prefix in excludes:
-                exclude = True
-        if exclude:
-            continue
-
         names = []
         format = {}
         for prefix in short:
+            if prefix in excludes:
+                continue
+
             for alias in [u.name] + [x for x in u.aliases if len(x) <= 2]:
                 names.append(prefix + alias)
 
@@ -1302,12 +1306,16 @@ def _add_prefixes(u, excludes=[], register=False):
                     format.setdefault(key, prefix + val)
 
         for prefix in long:
+            if prefix in excludes:
+                continue
+
             for alias in u.aliases:
                 if len(alias) > 2:
                     names.append(prefix + alias)
 
-        PrefixUnit(names, CompositeUnit(factor, [u], [1]), register=register,
-                   format=format)
+        if len(names):
+            PrefixUnit(names, CompositeUnit(factor, [u], [1]),
+                       register=register, format=format)
 
 
 def def_unit(s, represents=None, register=None, doc=None,
