@@ -31,6 +31,7 @@ class CDS(Base):
 
     @staticmethod
     def _generate_unit_names():
+        import keyword
         from ... import units as u
         names = {}
 
@@ -48,6 +49,8 @@ class CDS(Base):
         for base in bases:
             for prefix in prefixes:
                 key = prefix + base
+                if keyword.iskeyword(key):
+                    continue
                 names[key] = getattr(u, key)
 
         simple_bases = [
