@@ -47,14 +47,19 @@ def_unit(['deg', 'degree'], _numpy.pi / 180.0 * rad, register=True,
          doc="degree: angular measurement 1/360 of full rotation",
          format={'latex': r'{}^{\circ}', 'unicode': '°'})
 def_unit(['arcmin', 'arcminute'], 1.0 / 60.0 * deg, register=True,
+         prefixes=True,
          doc="arc minute: angular measurement",
-         format={'latex': r'\prime', 'unicode': '′'})
+         format={'latex': r'{}^{\prime}', 'unicode': '′'})
 def_unit(['arcsec', 'arcsecond'], 1.0 / 3600.0 * deg, register=True,
+         prefixes=True,
+         doc="arc second: angular measurement")
+# These special formats should only be used for the non-prefix versions
+arcsec._format = {'latex': r'{}^{\prime\prime}', 'unicode': '″'}
+def_unit(['mas'], 0.001 * arcsec, register=True,
+         doc="arc second: angular measurement")
+def_unit(['uas'], 0.000001 * arcsec, register=True,
          doc="arc second: angular measurement",
-         format={'latex': r'\second', 'unicode': '″'})
-def_unit(['mas'], 1.0 / 3600000 * deg, register=True,
-         doc="milliarc second: angular measurement",
-         format={'latex': r'\third', 'unicode': '‴'})
+         format={'latex': r'\mu as', 'unicode': 'μas'})
 
 def_unit(['sr', 'steradian'], rad ** 2, register=True, prefixes=True,
          doc="steradian: unit of solid angle in SI")
@@ -64,6 +69,7 @@ def_unit(['sr', 'steradian'], rad ** 2, register=True, prefixes=True,
 # TIME
 
 def_unit(['s', 'second'], register=True, prefixes=True,
+         exclude_prefixes=['a'],
          doc="second: base unit of time in SI.")
 
 def_unit(['min', 'minute'], 60 * s, register=True)
@@ -74,9 +80,9 @@ def_unit(['sday'], 86164.09053 * s, register=True,
 def_unit(['wk', 'week'], 7 * day, register=True)
 def_unit(['fortnight'], 2 * wk, register=True)
 
-def_unit(['a', 'annum'], 3.1556926e7 * s, register=True, prefixes=True,
+def_unit(['a', 'annum'], 365.25 * d, register=True, prefixes=True,
          exclude_prefixes=['P'])
-def_unit(['yr', 'year'], 3.1556926e7 * s, register=True, prefixes=True)
+def_unit(['yr', 'year'], 365.25 * d, register=True, prefixes=True)
 
 
 ###########################################################################
@@ -92,7 +98,7 @@ def_unit(['Hz', 'Hertz', 'hertz'], 1 / s, register=True, prefixes=True,
 def_unit(['kg', 'kilogram'], register=True,
          doc="kilogram: base unit of mass in SI.")
 def_unit(['g', 'gram'], 1.0e-3 * kg, register=True, prefixes=True,
-         exclude_prefixes=['k'])
+         exclude_prefixes=['k', 'kilo'])
 
 def_unit(['t', 'tonne'], 1000 * kg, register=True,
          doc="Metric tonne")
@@ -181,7 +187,7 @@ def_unit(['lx', 'lux'], lm * m ** -2, register=True, prefixes=True,
 ###########################################################################
 # BASES
 
-bases = set([m, s, kg, A, cd, rad, deg, K, mol])
+bases = set([m, s, kg, A, cd, rad, K, mol])
 
 
 ###########################################################################
