@@ -17,7 +17,8 @@ from ..config.configuration import ConfigurationItem
 __all__ = ['get_readable_fileobj', 'get_file_contents', 'get_pkg_data_fileobj',
            'get_pkg_data_filename', 'get_pkg_data_contents',
            'get_pkg_data_fileobjs', 'get_pkg_data_filenames', 'compute_hash',
-           'clear_download_cache', 'CacheMissingWarning', 'download_file',
+           'clear_download_cache', 'CacheMissingWarning',
+           'get_free_space_in_dir', 'check_free_space_in_dir', 'download_file',
            'download_files_in_parallel']
 
 DATAURL = ConfigurationItem(
@@ -70,9 +71,12 @@ def _is_url(string):
 
 
 def _is_inside(path, parent_path):
-    # We have to use realpath to avoid issues with symlinks
-    return os.path.realpath(path).startswith(os.path.realpath(parent_path))
-
+    # We have to try realpath too to avoid issues with symlinks, but we leave
+    # abspath because some systems like debian have the absolute path (with no
+    # symlinks followed) match, but the real directories in different
+    # locations, so need to try both cases.
+    return os.path.abspath(path).startswith(os.path.abspath(parent_path)) \
+        or os.path.realpath(path).startswith(os.path.realpath(parent_path))
 
 @contextlib.contextmanager
 def get_readable_fileobj(name_or_obj, encoding=None, cache=False):
@@ -722,6 +726,53 @@ def _find_hash_fn(hash):
         return None
 
 
+def get_free_space_in_dir(path):
+    """
+    Given a path to a directory, returns the amount of free space (in
+    bytes) on that filesystem.
+
+    Parameters
+    ----------
+    path : str
+        The path to a directory
+
+    Returns
+    -------
+    bytes : int
+        The amount of free space on the partition that the directory
+        is on.
+    """
+    stat = os.statvfs(path)
+    return stat.f_bavail * stat.f_frsize
+
+
+def check_free_space_in_dir(path, size):
+    """
+    Determines if a given directory has enough space to hold a file of
+    a given size.  Raises an IOError if the file would be too large.
+
+    Parameters
+    ----------
+    path : str
+        The path to a directory
+
+    size : int
+        A proposed filesize (in bytes)
+
+    Raises
+    -------
+    IOError : There is not enough room on the filesystem
+    """
+    from ..utils.console import human_file_size
+
+    space = get_free_space_in_dir(path)
+    if space < size:
+        raise IOError(
+            "Not enough free space in '{0}' "
+            "to download a {1} file".format(
+                path, human_file_size(size)))
+
+
 def download_file(remote_url, cache=False):
     """
     Accepts a URL, downloads and optionally caches the result
@@ -739,7 +790,7 @@ def download_file(remote_url, cache=False):
 
     import hashlib
     from contextlib import closing
-    from tempfile import NamedTemporaryFile
+    from tempfile import NamedTemporaryFile, gettempdir
     from shutil import move
     from warnings import warn
 
@@ -776,17 +827,27 @@ def download_file(remote_url, cache=False):
             else:
                 size = None
 
+            if size is not None:
+                check_free_space_in_dir(gettempdir(), size)
+                if cache:
+                    check_free_space_in_dir(dldir, size)
+
             dlmsg = "Downloading {0}".format(remote_url)
             with ProgressBarOrSpinner(size, dlmsg) as p:
-                with NamedTemporaryFile(delete=False) as f:
-                    bytes_read = 0
-                    block = remote.read(DOWNLOAD_CACHE_BLOCK_SIZE())
-                    while block:
-                        f.write(block)
-                        hash.update(block)
-                        bytes_read += len(block)
-                        p.update(bytes_read)
+                try:
+                    with NamedTemporaryFile(delete=False) as f:
+                        bytes_read = 0
                         block = remote.read(DOWNLOAD_CACHE_BLOCK_SIZE())
+                        while block:
+                            f.write(block)
+                            hash.update(block)
+                            bytes_read += len(block)
+                            p.update(bytes_read)
+                            block = remote.read(DOWNLOAD_CACHE_BLOCK_SIZE())
+                except:
+                    if os.path.exists(f.name):
+                        os.remove(f.name)
+                    raise
 
         if cache:
             _acquire_download_cache_lock()
@@ -812,7 +873,7 @@ def download_file(remote_url, cache=False):
                 global _tempfilestodel
                 _tempfilestodel.append(local_path)
     except urllib2.URLError as e:
-        if e.reason.errno == 8:
+        if hasattr(e, 'reason') and hasattr(e.reason, 'errno') and e.reason.errno == 8:
             e.reason.strerror = e.reason.strerror + '. requested URL: ' + remote_url
             e.reason.args = (e.reason.errno, e.reason.strerror)
         raise e
