@@ -9,14 +9,16 @@ from __future__ import absolute_import
 import collections
 import contextlib
 import functools
+import os
 import sys
 import textwrap
 import traceback
 import warnings
 
+
 __all__ = ['find_current_module', 'isiterable', 'deprecated', 'lazyproperty',
            'deprecated_attribute', 'silence', 'format_exception',
-           'NumpyRNGContext']
+           'NumpyRNGContext', 'is_path_hidden', 'walk_skip_hidden']
 
 
 def find_current_module(depth=1, finddiff=False):
@@ -289,7 +291,7 @@ class lazyproperty(object):
         args = [self._fget, self._fset, self._fdel, self.__doc__]
         args[arg] = f
         cls_ns = sys._getframe(1).f_locals
-        for k, v in cls_ns.iteritems():
+        for k, v in cls_ns.items():
             if v is self:
                 property_name = k
                 break
@@ -584,3 +586,85 @@ class NumpyRNGContext(object):
         from numpy import random
 
         random.set_state(self.startstate)
+
+
+def signal_number_to_name(signum):
+    """
+    Given an OS signal number, returns a signal name.  If the signal
+    number is unknown, returns ``'UNKNOWN'``.
+    """
+    # Since these numbers and names are platform specific, we use the
+    # builtin signal module and build a reverse mapping.
+
+    import signal
+    signal_to_name_map = dict(
+        (k, v) for v, k in signal.__dict__.iteritems() if v.startswith('SIG'))
+
+    return signal_to_name_map.get(signum, 'UNKNOWN')
+
+
+if sys.platform == 'win32':
+    import ctypes
+
+    def _has_hidden_attribute(filepath):
+        """
+        Returns True if the given filepath has the hidden attribute on
+        MS-Windows.  Based on a post here:
+        http://stackoverflow.com/questions/284115/cross-platform-hidden-file-detection
+        """
+        if isinstance(filepath, bytes):
+            filepath = filepath.decode(sys.getfilesystemencoding())
+        try:
+            attrs = ctypes.windll.kernel32.GetFileAttributesW(filepath)
+            assert attrs != -1
+            result = bool(attrs & 2)
+        except (AttributeError, AssertionError):
+            result = False
+        return result
+else:
+    def _has_hidden_attribute(filepath):
+        return False
+
+
+def is_path_hidden(filepath):
+    """
+    Determines if a given file or directory is hidden.
+
+    Parameters
+    ----------
+    filepath : str
+        The path to a file or directory
+
+    Returns
+    -------
+    hidden : bool
+        Returns `True` if the file is hidden
+    """
+    name = os.path.basename(os.path.abspath(filepath))
+    if isinstance(name, bytes):
+        is_dotted = name.startswith(b'.')
+    else:
+        is_dotted = name.startswith('.')
+    return is_dotted or _has_hidden_attribute(filepath)
+
+
+def walk_skip_hidden(top, onerror=None, followlinks=False):
+    """
+    A wrapper for `os.walk` that skips hidden files and directories.
+
+    This function does not have the parameter `topdown` from
+    `os.walk`: the directories must always be recursed top-down when
+    using this function.
+
+    See also
+    --------
+    os.walk : For a description of the parameters
+    """
+    for root, dirs, files in os.walk(
+            top, topdown=True, onerror=onerror,
+            followlinks=followlinks):
+        # These lists must be updated in-place so os.walk will skip
+        # hidden directories
+        dirs[:] = [d for d in dirs if not is_path_hidden(d)]
+        files[:] = [f for f in files if not is_path_hidden(f)]
+        yield root, dirs, files
