@@ -1363,11 +1363,11 @@ naxis kwarg.
           8. Keyword order may be changed.
         """
 
-        do_sip = (relax is True or
-                  relax == WCSHDO_all or
-                  (relax & WCSHDO_SIP))
         if relax not in (True, False):
+            do_sip = relax & WCSHDO_SIP
             relax &= ~WCSHDO_SIP
+        else:
+            do_sip = relax
 
         if self.wcs is not None:
             header_string = self.wcs.to_header(relax)
@@ -1386,7 +1386,7 @@ naxis kwarg.
         Identical to `to_header`, but returns a string containing the
         header cards.
         """
-        return str(self.to_header(self, relax))
+        return str(self.to_header(relax))
 
     def footprint_to_file(self, filename=None, color='green', width=2):
         """
